@@ -76,12 +76,10 @@ def get_ollama_response(
                         from npcpy.data.load import load_pdf
                         pdf_data = load_pdf(attachment)
                         if pdf_data is not None:
-                            texts = json.loads(pdf_data['texts'].iloc[0])
-                            pdf_text = "\n\n".join([item.get('content', '') for item in texts])
                             if prompt:
-                                prompt += f"\n\nContent from PDF: {os.path.basename(attachment)}\n{pdf_text[:5000]}..."
+                                prompt += f"\n\nContent from PDF: {os.path.basename(attachment)}\n{pdf_data[:5000]}..."
                             else:
-                                prompt = f"Content from PDF: {os.path.basename(attachment)}\n{pdf_text[:5000]}..."
+                                prompt = f"Content from PDF: {os.path.basename(attachment)}\n{pdf_data[:5000]}..."
                     except Exception:
                         pass
                 elif ext == '.csv':
@@ -242,7 +240,11 @@ def get_ollama_response(
         }
         
         # Process tool calls and get the updated result
-        processed_result = process_tool_calls(response_for_processing, tool_map, model, 'ollama', messages, stream=False)
+        processed_result = process_tool_calls(response_for_processing, 
+                                              tool_map, model, 
+                                              'ollama', 
+                                              messages, 
+                                              stream=False)
         
         # Now if streaming was requested, make a final call with the complete conversation
         if stream:
@@ -359,12 +361,11 @@ def get_litellm_response(
                         from npcpy.data.load import load_pdf
                         pdf_data = load_pdf(attachment)
                         if pdf_data is not None:
-                            texts = json.loads(pdf_data['texts'].iloc[0])
-                            pdf_text = "\n\n".join([item.get('content', '') for item in texts])
                             if prompt:
-                                prompt += f"\n\nContent from PDF: {os.path.basename(attachment)}\n{pdf_text[:5000]}..."
+                                prompt += f"\n\nContent from PDF: {os.path.basename(attachment)}\n{pdf_data[:5000]}..."
                             else:
-                                prompt = f"Content from PDF: {os.path.basename(attachment)}\n{pdf_text[:5000]}..."
+                                prompt = f"Content from PDF: {os.path.basename(attachment)}\n{pdf_data[:5000]}..."
+
                     except Exception:
                         pass
                 elif ext == '.csv':
@@ -381,18 +382,18 @@ def get_litellm_response(
                         pass
 
     if prompt:
-        if messages and messages[-1]["role"] == "user":
+        if result['messages'] and result['messages'][-1]["role"] == "user":
             if isinstance(messages[-1]["content"], str):
-                messages[-1]["content"] = prompt
-            elif isinstance(messages[-1]["content"], list):
-                for i, item in enumerate(messages[-1]["content"]):
+                result['messages'][-1]["content"] = prompt
+            elif isinstance(result['messages'][-1]["content"], list):
+                for i, item in enumerate(result['messages'][-1]["content"]):
                     if item.get("type") == "text":
-                        messages[-1]["content"][i]["text"] = prompt
+                        result['messages'][-1]["content"][i]["text"] = prompt
                         break
                 else:
-                    messages[-1]["content"].append({"type": "text", "text": prompt})
+                    result['messages'][-1]["content"].append({"type": "text", "text": prompt})
         else:
-            messages.append({"role": "user", "content": prompt})
+            result['messages'].append({"role": "user", "content": prompt})
 
     if format == "json" and not stream:
         json_instruction = """If you are a returning a json object, begin directly with the opening {.
@@ -477,7 +478,8 @@ def get_litellm_response(
             # Non-streaming response
             llm_response = resp.choices[0].message.content
             result["response"] = llm_response
-            result["messages"].append({"role": "assistant", "content": llm_response})
+            result["messages"].append({"role": "assistant", 
+                                       "content": llm_response})
             
             # Check for tool calls
             if hasattr(resp.choices[0].message, 'tool_calls') and resp.choices[0].message.tool_calls:
@@ -508,7 +510,7 @@ def get_litellm_response(
     initial_api_params = api_params.copy()
     initial_api_params["stream"] = False
     
-    print("Initial API params:", initial_api_params)
+    
     resp = completion(**initial_api_params)
     result["raw_response"] = resp
     
@@ -521,7 +523,12 @@ def get_litellm_response(
         result["tool_calls"] = resp.choices[0].message.tool_calls
         
         # Process tool calls
-        processed_result = process_tool_calls(result, tool_map, model, provider, result["messages"], stream=False)
+        processed_result = process_tool_calls(result, 
+                                              tool_map, 
+                                              model, 
+                                              provider, 
+                                              result["messages"], 
+                                              stream=False)
         
         # If streaming was requested, make final streaming call with processed conversation
         if stream:
@@ -580,12 +587,12 @@ def process_tool_calls(response_dict, tool_map, model, provider, messages, strea
     
     if not tool_calls:
         return result
-    print('tm', tool_map)
+    #print('tm', tool_map)
     for tool_call in tool_calls:
         tool_id = str(uuid.uuid4())
         tool_name = None
         arguments = {}
-        print('tc', tool_call)
+        #print('tc', tool_call)
 
         if isinstance(tool_call, dict):
             tool_id = tool_call.get("id", str(uuid.uuid4()))
@@ -604,8 +611,8 @@ def process_tool_calls(response_dict, tool_map, model, provider, messages, strea
             arguments = json.loads(arguments_str) if isinstance(arguments_str, str) else arguments_str
         except json.JSONDecodeError:
             arguments = {"raw_arguments": arguments_str}
-        print('arg', arguments)
-        print('tool name in tool map ', tool_name in tool_map)
+        #print('arg', arguments)
+        #print('tool name in tool map ', tool_name in tool_map)
         if tool_name in tool_map:
             tool_result = None
             tool_result_str = ""
