@@ -22,6 +22,7 @@ def gen_image(
     npc: Any = None,
     height: int = 1024,
     width: int = 1024,
+    n_images: int=1, 
     input_images: List[Union[str, bytes, PIL.Image.Image]] = None,
 ):
     """This function generates an image using the specified provider and model.
@@ -54,6 +55,7 @@ def gen_image(
         height=height,
         width=width, 
         attachments=input_images,
+        n_images=n_images, 
         
     )
     return image
@@ -204,8 +206,8 @@ def get_llm_response(
         if isinstance(messages[-1]["content"], str):
             messages[-1]["content"] += "\n" + prompt+context_str
     elif prompt:
-        messages.append({"role": "user", "content": prompt + context_str})
-
+        messages.append({"role": "user", 
+                         "content": prompt + context_str})    
     response = get_litellm_response(
         prompt + context_str,
         messages=messages,
@@ -397,8 +399,7 @@ def handle_jinx_call(
         #print(npc, team)
         #print(team.jinxs_dict, npc.jinxs_dict)
         if jinx_name not in npc.jinxs_dict and jinx_name not in team.jinxs_dict:
-            print("not available")
-            print(jinx_name, npc.jinxs_dict, team.jinxs_dict)
+            print(f"Jinx {jinx_name} not available")
             if attempt < n_attempts:
                 print(f"attempt {attempt+1} to generate jinx name failed, trying again")                
                 return check_llm_command(
@@ -418,8 +419,6 @@ def handle_jinx_call(
                     npc=npc,
                     team=team,
                     stream=stream,
-                    attempt=attempt + 1,
-                    n_attempts=n_attempts,
                     context=context
                 )
             return {
@@ -450,8 +449,13 @@ def handle_jinx_call(
 
         prompt = f"""
         The user wants to use the jinx '{jinx_name}' with the following request:
-        '{command}'
-        Here is the jinx file:
+        '{command}'"""
+
+
+        prompt += f'Here were the previous 5 messages in the conversation: {messages[-5:]}'
+
+
+        prompt+=f"""Here is the jinx file:
         ```
         {jinx.to_dict()}
         ```
@@ -465,13 +469,14 @@ def handle_jinx_call(
         If the jinx requires code to be generated, you must generate it exactly according to the instructions.
         Your inputs must satisfy the jinx's requirements.
 
+
+
+
         Return only the JSON object without any markdown formatting.
 
-        The format of the JSON object is:
-        {{
-        {json_format_str}
-        }}
-        """
+        The format of the JSON object is: 
+        
+        """+"{"+json_format_str+"}"
 
         if npc and hasattr(npc, "shared_context"):
             if npc.shared_context.get("dataframes"):
@@ -479,13 +484,13 @@ def handle_jinx_call(
                 for df_name in npc.shared_context["dataframes"].keys():
                     context_info += f"- {df_name}\n"
                 prompt += f"""Here is contextual info that may affect your choice: {context_info}
-                """
+                """                
         response = get_llm_response(
             prompt,
             format="json",
             model=model,
             provider=provider,
-
+            messages=messages[-10:], 
             npc=npc,
             context=context
         )
@@ -549,9 +554,6 @@ def handle_jinx_call(
                 npc=npc,
                 messages=messages,
             )
-            if 'llm_response' in jinx_output and 'messages' in jinx_output:
-                if len(jinx_output['llm_response'])>0:                
-                    messages = jinx_output['messages']
         except Exception as e:
             print(f"An error occurred while executing the jinx: {e}")
             print(f"trying again, attempt {attempt+1}")
@@ -570,12 +572,8 @@ def handle_jinx_call(
                     n_attempts=n_attempts,
                     context=f""" \n \n \n "jinx failed: {e}  \n \n \n here was the previous attempt: {input_values}""",
                 )
-        if not stream and messages[-1]['role'] != 'assistant':
-            # if the jinx has already added a message to the output from a final prompt we dont want to double that
-            
-            render_markdown(f""" ## jinx OUTPUT FROM CALLING {jinx_name} \n \n output:{jinx_output['output']}""" )
-
-            
+        if not stream and len(messages) > 0 :            
+            render_markdown(f""" ## jinx OUTPUT FROM CALLING {jinx_name} \n \n output:{jinx_output['output']}""" )            
             response = get_llm_response(f"""
                 The user had the following request: {command}. 
                 Here were the jinx outputs from calling {jinx_name}: {jinx_output}
@@ -585,9 +583,8 @@ def handle_jinx_call(
                 """,
                 model=model,
                 provider=provider,
-
                 npc=npc,
-                messages=messages,
+                messages=messages[-10:],
                 context=context, 
                 stream=stream,
             )
@@ -679,11 +676,12 @@ def answer_handler(command, extracted_data, **kwargs):
         provider=kwargs.get('provider'),
         api_url=kwargs.get('api_url'),
         api_key=kwargs.get('api_key'),
-        messages=kwargs.get('messages'),
+        messages=kwargs.get('messages',)[-10:],
         npc=kwargs.get('npc'),
         team=kwargs.get('team'), 
         stream=kwargs.get('stream', False),
-        images=kwargs.get('images')
+        images=kwargs.get('images'), 
+        context=kwargs.get('context')
     )
  
     return response
@@ -708,8 +706,7 @@ def check_llm_command(
 
     if actions is None:
         actions = DEFAULT_ACTION_SPACE.copy()
-
-    return execute_multi_step_plan(
+    exec =  execute_multi_step_plan(
         command=command,
         model=model,
         provider=provider,
@@ -724,6 +721,7 @@ def check_llm_command(
         actions=actions,
 
     )
+    return exec
 
 # Define `DEFAULT_ACTION_SPACE`
 
@@ -867,9 +865,7 @@ def plan_multi_step_actions(
     by dynamically building a prompt from the provided action space.
     """
     
-    # --- Start of the corrected logic ---
     
-    # 1. Build the prompt dynamically.
     prompt = f"""
 Analyze the user's request: "{command}"
 
@@ -878,13 +874,10 @@ Use the following context about available actions and tools to construct the pla
 
 """
 
-    # 2. Dynamically add context from the action space, executing lambdas as needed.
     for action_name, action_info in actions.items():
         ctx = action_info.get("context")
-        # If the context is a function (our lambda), call it to get the dynamic string.
         if callable(ctx):
             try:
-                # Pass the npc object to the lambda so it can access the jinxs_dict.
                 #print(ctx)
                 ctx = ctx(npc=npc, team=team)
             except Exception as e:
@@ -894,9 +887,9 @@ Use the following context about available actions and tools to construct the pla
         
         if ctx:
             prompt += f"\n--- Context for action '{action_name}' ---\n{ctx}\n"
+    if len(messages) >0:
+        prompt += f'Here were the previous 5 messages in the conversation: {messages[-5:]}'
 
-    #print(prompt)
-    # 3. Add the final instructions.
     prompt += f"""
 --- Instructions ---
 Based on the user's request and the context provided above, create a plan.
@@ -904,9 +897,6 @@ Based on the user's request and the context provided above, create a plan.
 The plan must be a JSON object with a single key, "actions". Each action must include:
 - "action": The name of the action to take.
 - "explanation": A clear description of the goal for this specific step.
-
-
-Example Request: "Find out who the current CEO of Microsoft is, then find their biography on Wikipedia"
  
 An Example Plan might look like this depending on the available actions:
 """ + """
@@ -926,7 +916,7 @@ An Example Plan might look like this depending on the available actions:
   ]
 }
 
-The plans should not mostly be 1-2 actions and usually never more than 3 actions at a time.
+The plans should mostly be 1-2 actions and usually never more than 3 actions at a time.
 Interactivity is important, unless a user specifies a usage of a specific action, it is generally best to
 assume just to respond in the simplest way possible rather than trying to assume certain actions have been requested.
 
@@ -944,7 +934,8 @@ Respond ONLY with the plan.
         npc=npc,
         team=team,
         format="json", 
-        messages=[]
+        messages=[], 
+        context=context,
     )
     response_content = action_response.get("response", {})
     #print(action_response)
@@ -999,7 +990,8 @@ def execute_multi_step_plan(
                                 npc=npc,
                                 stream=stream,
                                 team = team, 
-                                images=images)
+                                images=images, 
+                                context=context)
         return {"messages": result.get('messages',
                                        messages), 
                 "output": result.get('response')}
@@ -1007,9 +999,10 @@ def execute_multi_step_plan(
 
     step_outputs = []
     current_messages = messages.copy()
-    render_markdown(f"### Plan for Command: {command}")
+    render_markdown(f"### Plan for Command: {command[100:]}")
     for action in planned_actions:
-        step_info = json.dumps(action)
+        step_info = json.dumps({'action': action.get('action', ''), 
+                                'explanation': str(action.get('explanation',''))[0:10]+'...'})
         render_markdown(f'- {step_info}')
 
 
@@ -1037,25 +1030,25 @@ def execute_multi_step_plan(
            team=team,
            stream=stream, 
 
-           context=step_context, 
+           context=context+step_context, 
            images=images
         )
 
 
         action_output = result.get('output') or result.get('response')
-
+        
         if stream and len(planned_actions) > 1:
             # If streaming, we need to process the output with markdown rendering
             action_output = print_and_process_stream_with_markdown(action_output, model, provider)
         elif len(planned_actions) == 1:
             # If streaming and only one action, we can directly return the output
             # can circumvent because compile sequence results just returns single output results.
-            return {"messages": result.get('messages', current_messages), 
+            return {"messages": result.get('messages', 
+                                           current_messages), 
                     "output": action_output}
-        
-            
         step_outputs.append(action_output)        
-        current_messages = result.get('messages', current_messages)
+        current_messages = result.get('messages', 
+                                      current_messages)
 
     # render_markdown('## Reviewing output...')
     # need tot replace with a review step actually 
@@ -1239,8 +1232,8 @@ def identify_groups(
 def get_related_concepts_multi(node_name: str, 
                                node_type: str, 
                                all_concept_names, 
-                               model: str, 
-                               provider: str, 
+                               model: str = None,
+                               provider: str = None,
                                npc=None,
                                context : str = None, 
                                **kwargs):
@@ -1269,8 +1262,8 @@ def get_related_concepts_multi(node_name: str,
 def assign_groups_to_fact(
     fact: str,
     groups: List[str],
-    model,
-    provider,
+    model = None,
+    provider = None,
     npc = None, 
     context: str = None,
     **kwargs
@@ -1308,8 +1301,8 @@ def assign_groups_to_fact(
 def generate_group_candidates(
     items: List[str],
     item_type: str,
-    model: str,
-    provider: str,
+    model: str = None,
+    provider: str =None,
     npc = None,
     context: str = None,
     n_passes: int = 3,
@@ -1372,8 +1365,8 @@ def generate_group_candidates(
 
 def remove_idempotent_groups(
     group_candidates: List[str],
-    model: str,
-    provider: str,
+    model: str = None,
+    provider: str =None,
     npc = None, 
     context : str = None,
     **kwargs: Any
@@ -1436,7 +1429,7 @@ def breathe(
     messages: List[Dict[str, str]],
     model: str = None,
     provider: str = None, 
-    npc=  None,
+    npc =  None,
     context: str = None,
     **kwargs: Any
 ) -> Dict[str, Any]:
@@ -1467,10 +1460,10 @@ def breathe(
     Return a JSON like so:
 
     {
-        'high_level_objective': 'the overall goal so far for the user', 
-        'most_recent_task': 'The currently ongoing task', 
-        'accomplishments': ['accomplishment1', 'accomplishment2'], 
-        'failures': ['falures1', 'failures2'], 
+        "high_level_objective": "the overall goal so far for the user", 
+        "most_recent_task": "The currently ongoing task", 
+        "accomplishments": ["accomplishment1", "accomplishment2"], 
+        "failures": ["falures1", "failures2"], 
     }
 
     '''
@@ -1485,6 +1478,8 @@ def breathe(
                            **kwargs)
 
     res = result.get('response', {})
+    if isinstance(res, str):
+        raise Exception
     format_output = f"""Here is a summary of the previous session. 
     The high level objective was: {res.get('high_level_objective')} \n The accomplishments were: {res.get('accomplishments')}, 
     the failures were: {res.get('failures')} and the most recent task was: {res.get('most_recent_task')}   """
@@ -1753,7 +1748,12 @@ def get_facts(content_text,
 
         
 
-def zoom_in(facts, model, provider, npc=None, context: str = None, **kwargs):
+def zoom_in(facts, 
+            model= None,
+            provider=None, 
+            npc=None,
+            context: str = None, 
+            **kwargs):
     """Infer new implied facts from existing facts"""
     valid_facts = []
     for fact in facts:
@@ -1794,8 +1794,8 @@ def zoom_in(facts, model, provider, npc=None, context: str = None, **kwargs):
                                 **kwargs)
     return response["response"].get("implied_facts", [])
 def generate_groups(facts, 
-                    model, 
-                    provider, 
+                    model=None,
+                    provider=None,
                     npc=None,
                     context: str =None, 
                     **kwargs):
@@ -1835,8 +1835,8 @@ def generate_groups(facts,
     return response["response"].get("groups", [])
 
 def remove_redundant_groups(groups, 
-                            model, 
-                            provider, 
+                            model=None,
+                            provider=None,
                             npc=None,
                             context: str = None,
                             **kwargs):
@@ -1883,8 +1883,8 @@ def remove_redundant_groups(groups,
 
 def prune_fact_subset_llm(fact_subset, 
                           concept_name, 
-                          model, 
-                          provider, 
+                          model=None,
+                          provider=None,
                           npc=None,
                           context : str = None,
                           **kwargs):
@@ -2017,8 +2017,11 @@ def find_best_link_concept_llm(candidate_concept_name,
 
 def asymptotic_freedom(parent_concept, 
                        supporting_facts, 
-                       model, provider, npc = None,
-                       context: str = None, **kwargs):
+                       model=None, 
+                       provider=None, 
+                       npc = None,
+                       context: str = None, 
+                       **kwargs):
     """Given a concept and its facts, proposes an intermediate layer of sub-concepts."""
     print(f"  Step Sleep-B: Attempting to deepen concept '{parent_concept['name']}'...")
     fact_statements = []
