@@ -29,6 +29,9 @@ except NameError as e:
     print('name error importing chromadb:', e)
     chromadb = None
 
+
+import logging 
+
 def flush_messages(n: int, messages: list) -> dict:
     if n <= 0:
         return {
@@ -450,7 +453,8 @@ class CommandHistory:
             Column('attachment_type', String(100)),
             Column('attachment_data', LargeBinary),
             Column('attachment_size', Integer),
-            Column('upload_timestamp', String(50))
+            Column('upload_timestamp', String(50)),
+            Column('file_path', Text) 
         )
         
         # Jinx execution log table
@@ -533,81 +537,79 @@ class CommandHistory:
         return str(uuid.uuid4())
 
     def add_conversation(
-        self, role, content, conversation_id, directory_path,
-        model=None, provider=None, npc=None, team=None,
-        attachments=None, message_id=None,
+        self, 
+        message_id,
+        timestamp,
+        role, 
+        content, 
+        conversation_id, 
+        directory_path,
+        model=None, 
+        provider=None, 
+        npc=None, 
+        team=None,
+        attachments=None,
     ):
-        timestamp = datetime.now().strftime("%Y-%m-%d %H:%M:%S")
-        if message_id is None:
-            message_id = self.generate_message_id()
-        if isinstance(content, dict):
+        if isinstance(content, (dict, list)):
             content = json.dumps(content, cls=CustomJSONEncoder)
 
-        # Check if message exists
-        existing = self._fetch_one(
-            "SELECT content FROM conversation_history WHERE message_id = :message_id",
-            {"message_id": message_id}
-        )
-
-        if existing:
-            # Update existing message
-            stmt = """
-                UPDATE conversation_history 
-                SET content = :content, timestamp = :timestamp 
-                WHERE message_id = :message_id
-            """
-            params = {"content": content, "timestamp": timestamp, "message_id": message_id}
-        else:
-            # Insert new message
-            stmt = """
-                INSERT INTO conversation_history
-                (message_id, timestamp, role, content, conversation_id, directory_path, model, provider, npc, team)
-                VALUES (:message_id, :timestamp, :role, :content, :conversation_id, :directory_path, :model, :provider, :npc, :team)
-            """
-            params = {
-                "message_id": message_id, "timestamp": timestamp, "role": role, "content": content,
-                "conversation_id": conversation_id, "directory_path": directory_path,
-                "model": model, "provider": provider, "npc": npc, "team": team
-            }
-
+        stmt = """
+            INSERT INTO conversation_history
+            (message_id, timestamp, role, content, conversation_id, directory_path, model, provider, npc, team)
+            VALUES (:message_id, :timestamp, :role, :content, :conversation_id, :directory_path, :model, :provider, :npc, :team)
+        """
+        params = {
+            "message_id": message_id, "timestamp": timestamp, "role": role, "content": content,
+            "conversation_id": conversation_id, "directory_path": directory_path, "model": model,
+            "provider": provider, "npc": npc, "team": team
+        }
         with self.engine.begin() as conn:
             conn.execute(text(stmt), params)
 
         if attachments:
             for attachment in attachments:
                 self.add_attachment(
-                    message_id, attachment["name"], attachment["type"],
-                    attachment["data"], attachment_size=attachment.get("size"),
+                    message_id=message_id,
+                    name=attachment.get("name"),
+                    attachment_type=attachment.get("type"),
+                    data=attachment.get("data"),
+                    size=attachment.get("size"),
+                    file_path=attachment.get("path") # PASS THE PATH
                 )
+
         return message_id
 
-    def add_attachment(self, message_id, attachment_name, attachment_type, attachment_data, attachment_size=None):
+    def add_attachment(self, message_id, name, attachment_type, data, size, file_path=None):
         timestamp = datetime.now().strftime("%Y-%m-%d %H:%M:%S")
-        if attachment_size is None and attachment_data is not None:
-            attachment_size = len(attachment_data)
-        
         stmt = """
-            INSERT INTO message_attachments
-            (message_id, attachment_name, attachment_type, attachment_data, attachment_size, upload_timestamp)
-            VALUES (:message_id, :attachment_name, :attachment_type, :attachment_data, :attachment_size, :upload_timestamp)
+            INSERT INTO message_attachments 
+            (message_id, attachment_name, attachment_type, attachment_data, attachment_size, upload_timestamp, file_path)
+            VALUES (:message_id, :name, :type, :data, :size, :timestamp, :file_path)
         """
         params = {
             "message_id": message_id,
-            "attachment_name": attachment_name,
-            "attachment_type": attachment_type,
-            "attachment_data": attachment_data,
-            "attachment_size": attachment_size,
-            "upload_timestamp": timestamp
+            "name": name,
+            "type": attachment_type,
+            "data": data,
+            "size": size,
+            "timestamp": timestamp,
+            "file_path": file_path
         }
-        
         with self.engine.begin() as conn:
             conn.execute(text(stmt), params)
 
     def save_jinx_execution(
-        self, triggering_message_id: str, conversation_id: str, npc_name: Optional[str],
-        jinx_name: str, jinx_inputs: Dict, jinx_output: Any, status: str,
-        team_name: Optional[str] = None, error_message: Optional[str] = None,
-        response_message_id: Optional[str] = None, duration_ms: Optional[int] = None
+        self, 
+        triggering_message_id: str, 
+        conversation_id: str, 
+        npc_name: Optional[str],
+        jinx_name: str, 
+        jinx_inputs: Dict, 
+        jinx_output: Any, status: str,
+        team_name: Optional[str] = None, 
+        error_message: Optional[str] = None,
+        response_message_id: Optional[str] = None, 
+        duration_ms: Optional[int] = None
     ):
         timestamp = datetime.now().strftime("%Y-%m-%d %H:%M:%S")
         
@@ -660,7 +662,13 @@ class CommandHistory:
 
     def get_message_attachments(self, message_id) -> List[Dict]:
         stmt = """
-            SELECT id, message_id, attachment_name, attachment_type, attachment_size, upload_timestamp
+            SELECT 
+                id, 
+                message_id, 
+                attachment_name, 
+                attachment_type, 
+                attachment_size, 
+                upload_timestamp
             FROM message_attachments WHERE message_id = :message_id
         """
         return self._fetch_all(stmt, {"message_id": message_id})
@@ -893,7 +901,7 @@ class CommandHistory:
         if self.engine:
             try:
                 self.engine.dispose()
-                print("Disposed SQLAlchemy engine.")
+                logging.info("Disposed SQLAlchemy engine.")
             except Exception as e:
                 print(f"Error disposing SQLAlchemy engine: {e}")
         self.engine = None
@@ -924,9 +932,12 @@ def save_conversation_message(
     """
     if wd is None:
         wd = os.getcwd()
+    timestamp = datetime.now().strftime("%Y-%m-%d %H:%M:%S")
+
 
     return command_history.add_conversation(
         role=role,
+        timestamp=timestamp, 
         content=content,
         conversation_id=conversation_id,
         directory_path=wd,
