@@ -108,7 +108,8 @@ class Jinx:
         context.update({
             "jinxs": jinxs_dict,
             "llm_response": None,
-            "output": None
+            "output": None, 
+            "messages": messages,
         })
         
         # Process each step in sequence
@@ -156,7 +157,7 @@ class Jinx:
         if rendered_engine == "natural":
             if rendered_code.strip():
                 # Handle streaming case
-                response =  npc.get_llm_response(
+                response = npc.get_llm_response(
                     rendered_code,
                     context=context,
                     messages=messages,
@@ -198,8 +199,13 @@ class Jinx:
             
             # Handle explicit output
             if "output" in exec_locals:
-                context["output"] = exec_locals["output"]
-                context[step_name] = exec_locals["output"]
+                outp = exec_locals["output"]
+                context["output"] = outp
+                context[step_name] = outp
+                messages.append({'role':'assistant', 
+                                 'content': f'Jinx executed with following output: {outp}'})
+                context['messages'] = messages
+                
         else:
             # Handle unknown engine
             context[step_name] = {"error": f"Unsupported engine: {rendered_engine}"}
@@ -1652,23 +1658,38 @@ def initialize_npc_project(
     npc_team_dir = os.path.join(directory, "npc_team")
     os.makedirs(npc_team_dir, exist_ok=True)
     
-    for subdir in ["jinxs", "assembly_lines", "sql_models", "jobs"]:
+    for subdir in ["jinxs", 
+                   "assembly_lines", 
+                   "sql_models", 
+                   "jobs", 
+                   "triggers"]:
         os.makedirs(os.path.join(npc_team_dir, subdir), exist_ok=True)
     
-    forenpc_path = os.path.join(npc_team_dir, "sibiji.npc")
+    forenpc_path = os.path.join(npc_team_dir, "forenpc.npc")
     
 
     # Always ensure default NPC exists
     if not os.path.exists(forenpc_path):
         # Use your new NPC class to create sibiji
         default_npc = {
-            "name": "sibiji",
-            "primary_directive": "You are sibiji, the forenpc of an NPC team...",
-            "model": model or "llama3.2",
-            "provider": provider or "ollama"
+            "name": "forenpc",
+            "primary_directive": "You are the forenpc of an NPC team", 
         }
         with open(forenpc_path, "w") as f:
             yaml.dump(default_npc, f)
+    ctx_path = os.path.join(npc_team_dir, "team.ctx")
+    if not os.path.exists(ctx_path):
+        default_ctx = {
+            'name': '',
+            'context' : '', 
+            'preferences': '', 
+            'mcp_servers': '', 
+            'databases':'', 
+            'use_global_jinxs': True,
+            'forenpc': 'forenpc'
+        }
+        with open(ctx_path, "w") as f:
+            yaml.dump(default_ctx, f)
             
     return f"NPC project initialized in {npc_team_dir}"
 
