@@ -7,9 +7,10 @@ import socket
 import concurrent.futures # For managing timeouts on blocking calls
 import platform
 import sqlite3
+import sys
 from typing import Dict, List
 import textwrap
-
+import json
 ON_WINDOWS = platform.system() == "Windows"
 
 # Try/except for termios, tty, pty, select, signal
@@ -95,7 +96,7 @@ def get_locally_available_models(project_directory, airplane_mode=False):
     # --- End internet check ---
 
     if not airplane_mode:
-        timeout_seconds = 0.5
+        timeout_seconds = 3.5
         # Use a single ThreadPoolExecutor for all network-dependent API calls
         with concurrent.futures.ThreadPoolExecutor(max_workers=1) as executor:
 
@@ -123,8 +124,6 @@ def get_locally_available_models(project_directory, airplane_mode=False):
                     import openai
 
                     def fetch_openai_models():
-                        # For older OpenAI client versions (<1.0.0), global api_key is common.
-                        # For openai>=1.0.0, it's client = openai.OpenAI(api_key=..., timeout=timeout_seconds)
                         openai.api_key = env_vars.get("OPENAI_API_KEY", None) or os.environ.get("OPENAI_API_KEY", None)
                         return openai.models.list()
 
@@ -148,20 +147,29 @@ def get_locally_available_models(project_directory, airplane_mode=False):
 
             if "GEMINI_API_KEY" in env_vars or os.environ.get("GEMINI_API_KEY"):
                 try:
-                    from google import generativeai as genai
-                
+                    from google import genai
                     def fetch_gemini_models():
-                        client = genai.Client(api_key = env_vars.get("GEMINI_API_KEY") or os.environ.get("GEMINI_API_KEY"))
+                        client = genai.Client(api_key=env_vars.get("GEMINI_API_KEY") or os.environ.get("GEMINI_API_KEY"))
                         found_models = []
-                        # client.models.list() returns an iterator, consume it fully within the callable
+                        # Define the model names you want to filter for
+                        target_models = [
+                            'gemini-2.5-pro', 
+                            'gemini-2.5-flash', 
+                            'gemini-2.0-flash', 
+                            'gemini-2.0-pro', 
+                            'gemini-1.5-pro', 
+                            'gemini-1.5-flash'
+                        ]
+                        
                         for m in client.models.list():
                             for action in m.supported_actions:
                                 if action == "generateContent":
                                     if 'models/' in m.name:
-                                        if m.name.split('/')[1] in ['gemini-2.5-pro', 'gemini-2.5-flash', 'gemini-2.0-flash', 'gemini-2.0-pro', 'gemini-1.5-pro', 'gemini-1.5-flash']:
-                                            found_models.append(m.name.split('/')[1])
+                                        model_name_part = m.name.split('/')[1]  # Extract the part after 'models/'
+                                        # Check if any of the target models are contained within the model name
+                                        if any(model in model_name_part for model in target_models):
+                                            found_models.append(model_name_part)
                         return set(found_models)
-                    
                     future = executor.submit(fetch_gemini_models)
                     models = future.result(timeout=timeout_seconds) # Apply timeout
 
@@ -172,13 +180,8 @@ def get_locally_available_models(project_directory, airplane_mode=False):
                     logging.info(f"Gemini models not indexed or timed out: {e}")
             
             if "DEEPSEEK_API_KEY" in env_vars or os.environ.get("DEEPSEEK_API_KEY"):
-                # Assuming Deepseek models are just added if the key exists, without an API call
-                # If an API call is needed here, apply similar timeout logic.
                 available_models['deepseek-chat'] = 'deepseek'
                 available_models['deepseek-reasoner'] = 'deepseek'        
-    
-    # Ollama is typically local, but its `list()` call might still involve a network request
-    # to the local Ollama server, so applying a timeout is still good practice.
     try:
         import ollama
         timeout_seconds = 0.5 # Re-using the same timeout
@@ -462,7 +465,9 @@ def render_code_block(code: str, language: str = None) -> None:
         code, language or "python", theme="monokai", line_numbers=False, padding=0
     )
     console.print(syntax)
-def print_and_process_stream_with_markdown(response, model, provider):
+    
+def print_and_process_stream_with_markdown(response, model, provider, show=False):
+    import sys
     
     str_output = ""
     dot_count = 0  
@@ -474,12 +479,13 @@ def print_and_process_stream_with_markdown(response, model, provider):
         print('\n') 
         return response 
     
+    # Save cursor position at the start
+    sys.stdout.write('\033[s')  # Save cursor position
+    sys.stdout.flush()
+    
     try:
         for chunk in response:
 
-            print('.', end="", flush=True)
-            dot_count += 1
-
             if provider == "ollama" and 'gpt-oss' not in model:
 
                 if "message" in chunk and "tool_calls" in chunk["message"]:
@@ -490,13 +496,22 @@ def print_and_process_stream_with_markdown(response, model, provider):
                             if "name" in tool_call["function"]:
                                 tool_call_data["function_name"] = tool_call["function"]["name"]
                             if "arguments" in tool_call["function"]:
-                                tool_call_data["arguments"] += tool_call["function"]["arguments"]
-                
+                                if isinstance(tool_call["function"]["arguments"], dict):
+                                    tool_call_data["arguments"] += json.dumps(tool_call["function"]["arguments"])
+                                else:
+                                    tool_call_data["arguments"] += tool_call["function"]["arguments"]                
                 chunk_content = chunk["message"]["content"] if "message" in chunk and "content" in chunk["message"] else ""
-                reasoning_content = chunk['message'].get('thinking')
+                reasoning_content = chunk['message'].get('thinking', '') if "message" in chunk and "thinking" in chunk['message'] else ""
+                if show:
+                    if len(reasoning_content) > 0:
+                        print(reasoning_content, end="", flush=True)
+                    if chunk_content != "":
+                        print(chunk_content, end="", flush=True)
+                else:
+                    print('.', end="", flush=True)
+                    dot_count += 1
+                    
             else:
-                
-
                 for c in chunk.choices:
                     if hasattr(c.delta, "tool_calls") and c.delta.tool_calls:
                         for tool_call in c.delta.tool_calls:
@@ -514,14 +529,21 @@ def print_and_process_stream_with_markdown(response, model, provider):
                     if hasattr(c.delta, "reasoning_content"):        
                         reasoning_content += c.delta.reasoning_content
                 
-                if reasoning_content:
+                if len(reasoning_content) > 0:
                     chunk_content = reasoning_content
                         
                 chunk_content += "".join(
                     c.delta.content for c in chunk.choices if c.delta.content
                 )
+                if show:
+                    if reasoning_content is not None:
+                        print(reasoning_content, end="", flush=True)
+                    if chunk_content != "":
+                        print(chunk_content, end="", flush=True)
+                else:
+                    print('.', end="", flush=True)
+                    dot_count += 1
 
-            
             if not chunk_content:
                 continue
             str_output += chunk_content
@@ -530,57 +552,139 @@ def print_and_process_stream_with_markdown(response, model, provider):
         interrupted = True
         print('\n⚠️ Stream interrupted by user')
     
-    print('\r' + ' ' * dot_count*2 + '\r', end="", flush=True)
-    
-
     if tool_call_data["id"] or tool_call_data["function_name"] or tool_call_data["arguments"]:
-        str_output += "\n\n### Jinx Call Data\n"
+        str_output += "\n\n### Tool Call Data\n"
         if tool_call_data["id"]:
             str_output += f"**ID:** {tool_call_data['id']}\n\n"
         if tool_call_data["function_name"]:
             str_output += f"**Function:** {tool_call_data['function_name']}\n\n"
         if tool_call_data["arguments"]:
             try:
-                import json
                 args_parsed = json.loads(tool_call_data["arguments"])
                 str_output += f"**Arguments:**\n```json\n{json.dumps(args_parsed, indent=2)}\n```"
             except:
                 str_output += f"**Arguments:** `{tool_call_data['arguments']}`"
-    
 
     if interrupted:
         str_output += "\n\n[⚠️ Response interrupted by user]"
     
+    # Always restore cursor position and clear everything after it
+    sys.stdout.write('\033[u')  # Restore cursor position
+    sys.stdout.write('\033[J')  # Clear from cursor down
+    sys.stdout.flush()
+    
+    # Now render the markdown at the restored position
+    render_markdown(str_output)
     print('\n')
-    render_markdown('\n' + str_output)
     
     return str_output
 
 
-
 def print_and_process_stream(response, model, provider):
-    conversation_result = ""
     
-    for chunk in response:
-        if provider == "ollama" and 'hf.co' in model:
-            chunk_content = chunk["message"]["content"]
-            if chunk_content:
-                conversation_result += chunk_content
-                print(chunk_content, end="")
+    str_output = ""
+    dot_count = 0  
+    tool_call_data = {"id": None, "function_name": None, "arguments": ""}
+    interrupted = False
+    
+    thinking_part=True
+    thinking_str=''
+    if isinstance(response, str):
+        render_markdown(response)  
+        print('\n') 
+        return response 
+    try:
+        for chunk in response:
 
-        else:
-            chunk_content = "".join(
-                choice.delta.content
-                for choice in chunk.choices
-                if choice.delta.content is not None
-            )
-            if chunk_content:
-                conversation_result += chunk_content
-                print(chunk_content, end="")
+            if provider == "ollama" and 'gpt-oss' not in model:
 
-    print("\n")
+                if "message" in chunk and "tool_calls" in chunk["message"]:
+                    for tool_call in chunk["message"]["tool_calls"]:
+                        if "id" in tool_call:
+                            tool_call_data["id"] = tool_call["id"]
+                        if "function" in tool_call:
+                            if "name" in tool_call["function"]:
+                                tool_call_data["function_name"] = tool_call["function"]["name"]
+                            if "arguments" in tool_call["function"]:
+                                if isinstance(tool_call["function"]["arguments"], dict):
+                                    tool_call_data["arguments"] += json.dumps(tool_call["function"]["arguments"])
+                                else:
+                                    tool_call_data["arguments"] += tool_call["function"]["arguments"]                
+                chunk_content = chunk["message"]["content"] if "message" in chunk and "content" in chunk["message"] else ""
+                reasoning_content = chunk['message'].get('thinking', '') if "message" in chunk and "thinking" in chunk['message'] else ""
+
+                if len(reasoning_content) > 0:
+                    print(reasoning_content, end="", flush=True)
+                    thinking_part = True
+                if chunk_content != "":
+                    print(chunk_content, end="", flush=True)
+                    
+            else:
+                for c in chunk.choices:
+                    if hasattr(c.delta, "tool_calls") and c.delta.tool_calls:
+                        for tool_call in c.delta.tool_calls:
+                            if tool_call.id:
+                                tool_call_data["id"] = tool_call.id
+                            if tool_call.function:
+                                if hasattr(tool_call.function, "name") and tool_call.function.name:
+                                    tool_call_data["function_name"] = tool_call.function.name
+                                if hasattr(tool_call.function, "arguments") and tool_call.function.arguments:
+                                    tool_call_data["arguments"] += tool_call.function.arguments
                 
-    return conversation_result   
+                chunk_content = ''
+                reasoning_content = ''
+                for c in chunk.choices:
+                    if hasattr(c.delta, "reasoning_content"):        
+                        reasoning_content += c.delta.reasoning_content
+                
+                        
+                chunk_content += "".join(
+                    c.delta.content for c in chunk.choices if c.delta.content
+                )
+                if reasoning_content is not None:
+                    if thinking_part:
+                        thinking_str +='<think>'
+                        thinking_part=False
+                        print('<think>')
+                    print(reasoning_content, end="", flush=True)
+                    thinking_str+=reasoning_content
+                # chunk content wont be nonzero length until thinking part is done.
+                
+                if chunk_content != "":
+                    if len(thinking_str) >0 and not thinking_part and '</think>' not in thinking_str:
+
+                        thinking_str+='</think>'
+                        print('</think>')
+                    print(chunk_content, end="", flush=True)
+
+
+            if not chunk_content:
+                continue
+            str_output += chunk_content
+    
+    except KeyboardInterrupt:
+        interrupted = True
+        print('\n⚠️ Stream interrupted by user')
+    
+    if tool_call_data["id"] or tool_call_data["function_name"] or tool_call_data["arguments"]:
+        str_output += "\n\n### Tool Call Data\n"
+        if tool_call_data["id"]:
+            str_output += f"**ID:** {tool_call_data['id']}\n\n"
+        if tool_call_data["function_name"]:
+            str_output += f"**Function:** {tool_call_data['function_name']}\n\n"
+        if tool_call_data["arguments"]:
+            try:
+                args_parsed = json.loads(tool_call_data["arguments"])
+                str_output += f"**Arguments:**\n```json\n{json.dumps(args_parsed, indent=2)}\n```"
+            except:
+                str_output += f"**Arguments:** `{tool_call_data['arguments']}`"
+
+    if interrupted:
+        str_output += "\n\n[⚠️ Response interrupted by user]"
+    
+
+                
+    return thinking_str+str_output   
 def get_system_message(npc, team=None) -> str:
     import os
     from datetime import datetime
